@@ -5,12 +5,14 @@ import os
 from os import environ
 from os import getpid
 import sys
+from threading import RLock
 from typing import Any
 from typing import Callable
 from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Set
+from typing import TypeVar
 from typing import Union
 
 from ddtrace import config
@@ -27,12 +29,13 @@ from .context import Context
 from .ext import system
 from .ext.priority import AUTO_KEEP
 from .ext.priority import AUTO_REJECT
-from .internal import _rand
 from .internal import agent
 from .internal import atexit
 from .internal import compat
 from .internal import debug
+from .internal import forksafe
 from .internal import hostname
+from .internal import service
 from .internal.dogstatsd import get_dogstatsd_client
 from .internal.logger import get_logger
 from .internal.logger import hasHandlers
@@ -40,6 +43,7 @@ from .internal.processor import SpanProcessor
 from .internal.processor.trace import SpanAggregator
 from .internal.processor.trace import TraceProcessor
 from .internal.processor.trace import TraceSamplingProcessor
+from .internal.processor.trace import TraceTagsProcessor
 from .internal.runtime import get_runtime_id
 from .internal.writer import AgentWriter
 from .internal.writer import LogWriter
@@ -59,12 +63,13 @@ from .utils.formats import get_env
 log = get_logger(__name__)
 
 debug_mode = asbool(get_env("trace", "debug", default=False))
+call_basic_config = asbool(os.environ.get("DD_CALL_BASIC_CONFIG", "true"))
 
 DD_LOG_FORMAT = "%(asctime)s %(levelname)s [%(name)s] [%(filename)s:%(lineno)d] {}- %(message)s".format(
     "[dd.service=%(dd.service)s dd.env=%(dd.env)s dd.version=%(dd.version)s"
     " dd.trace_id=%(dd.trace_id)s dd.span_id=%(dd.span_id)s] "
 )
-if debug_mode and not hasHandlers(log):
+if debug_mode and not hasHandlers(log) and call_basic_config:
     if config.logs_injection:
         logging.basicConfig(level=logging.DEBUG, format=DD_LOG_FORMAT)
     else:
@@ -74,6 +79,9 @@ if debug_mode and not hasHandlers(log):
 _INTERNAL_APPLICATION_SPAN_TYPES = {"custom", "template", "web", "worker"}
 
 
+AnyCallable = TypeVar("AnyCallable", bound=Callable)
+
+
 class Tracer(object):
     """
     Tracer is used to create, sample and submit spans that measure the
@@ -142,6 +150,11 @@ class Tracer(object):
         self._initialize_span_processors()
         self._hooks = _hooks.Hooks()
         atexit.register(self._atexit)
+        forksafe.register(self._child_after_fork)
+
+        self._shutdown_lock = RLock()
+
+        self._new_process = False
 
     def _atexit(self):
         # type: () -> None
@@ -195,6 +208,9 @@ class Tracer(object):
     def global_excepthook(self, tp, value, traceback):
         """The global tracer except hook."""
 
+    @deprecated(
+        "Call context has been superseded by trace context. Please use current_trace_context() instead.", "1.0.0"
+    )
     def get_call_context(self, *args, **kwargs):
         # type: (...) -> Context
         """
@@ -212,10 +228,25 @@ class Tracer(object):
         This method makes use of a ``ContextProvider`` that is automatically set during the tracer
         initialization, or while using a library instrumentation.
         """
-        return self.context_provider.active(*args, **kwargs)  # type: ignore
+        ctx = self.current_trace_context(*args, **kwargs)
+        if ctx is None:
+            ctx = Context()
+        return ctx
+
+    def current_trace_context(self, *args, **kwargs):
+        # type (...) -> Optional[Context]
+        """Return the context for the current trace.
+
+        If there is no active trace then None is returned.
+        """
+        active = self.context_provider.active(*args, **kwargs)
+        if isinstance(active, Context):
+            return active
+        elif isinstance(active, Span):
+            return active.context
+        return None
 
     # TODO: deprecate this method and make sure users create a new tracer if they need different parameters
-    @debtcollector.removals.removed_kwarg("collect_metrics", removal_version="0.51")
     def configure(
         self,
         enabled=None,  # type: Optional[bool]
@@ -228,7 +259,6 @@ class Tracer(object):
         wrap_executor=None,  # type: Optional[Callable]
         priority_sampling=None,  # type: Optional[bool]
         settings=None,  # type: Optional[Dict[str, Any]]
-        collect_metrics=None,  # type: Optional[bool]
         dogstatsd_url=None,  # type: Optional[str]
         writer=None,  # type: Optional[TraceWriter]
         partial_flush_enabled=None,  # type: Optional[bool]
@@ -254,7 +284,6 @@ class Tracer(object):
             from the default value
         :param priority_sampling: enable priority sampling, this is required for
             complete distributed tracing support. Enabled by default.
-        :param collect_metrics: Whether to enable runtime metrics collection.
         :param str dogstatsd_url: URL for UDP or Unix socket connection to DogStatsD
         """
         if enabled is not None:
@@ -312,7 +341,11 @@ class Tracer(object):
             # get the URL from.
             url = None  # type: ignore
 
-        self.writer.stop()
+        try:
+            self.writer.stop()
+        except service.ServiceStatusError:
+            # It's possible the writer never got started in the first place :(
+            pass
 
         if writer is not None:
             self.writer = writer
@@ -331,7 +364,7 @@ class Tracer(object):
             # No need to do anything for the LogWriter.
             pass
         if isinstance(self.writer, AgentWriter):
-            self.writer.dogstatsd = get_dogstatsd_client(self._dogstatsd_url)
+            self.writer.dogstatsd = get_dogstatsd_client(self._dogstatsd_url)  # type: ignore[has-type]
         self._initialize_span_processors()
 
         if context_provider is not None:
@@ -340,18 +373,6 @@ class Tracer(object):
         if wrap_executor is not None:
             self._wrap_executor = wrap_executor
 
-        runtime_metrics_was_running = False
-        # FIXME: Import RuntimeWorker here to avoid circular imports. This will
-        # be gone together with the collect_metrics attribute soon.
-        from .internal.runtime.runtime_metrics import RuntimeWorker
-
-        if RuntimeWorker._instance is not None:
-            runtime_metrics_was_running = True
-            RuntimeWorker.disable()
-
-        if (collect_metrics is None and runtime_metrics_was_running) or collect_metrics:
-            RuntimeWorker.enable(tracer=self, dogstatsd_url=self._dogstatsd_url)
-
         if debug_mode or asbool(environ.get("DD_TRACE_STARTUP_LOGS", False)):
             try:
                 info = debug.collect(self)
@@ -370,59 +391,101 @@ class Tracer(object):
                     msg = "- DATADOG TRACER DIAGNOSTIC - %s" % agent_error
                     self._log_compat(logging.WARNING, msg)
 
-    def start_span(
+    def _child_after_fork(self):
+        self._pid = getpid()
+
+        # Assume that the services of the child are not necessarily a subset of those
+        # of the parent.
+        self._services = set()
+
+        # Re-create the background writer thread
+        self.writer = self.writer.recreate()
+        self._initialize_span_processors()
+
+        self._new_process = True
+
+    def _start_span(
         self,
         name,  # type: str
         child_of=None,  # type: Optional[Union[Span, Context]]
         service=None,  # type: Optional[str]
         resource=None,  # type: Optional[str]
         span_type=None,  # type: Optional[str]
+        activate=False,  # type: bool
     ):
         # type: (...) -> Span
-        """
-        Return a span that will trace an operation called `name`. This method allows
-        parenting using the ``child_of`` kwarg. If it's missing, the newly created span is a
-        root span.
+        """Return a span that represents an operation called ``name``.
+
+        Note that the :meth:`.trace` method will almost always be preferred
+        over this method as it provides automatic span parenting. This method
+        should only be used if manual parenting is desired.
 
         :param str name: the name of the operation being traced.
         :param object child_of: a ``Span`` or a ``Context`` instance representing the parent for this span.
         :param str service: the name of the service being traced.
         :param str resource: an optional name of the resource being tracked.
         :param str span_type: an optional operation type.
+        :param activate: activate the span once it is created.
 
-        To start a new root span, simply::
+        To start a new root span::
 
-            span = tracer.start_span('web.request')
+            span = tracer.start_span("web.request")
 
-        If you want to create a child for a root span, just::
+        To create a child for a root span::
 
-            root_span = tracer.start_span('web.request')
-            span = tracer.start_span('web.decoder', child_of=root_span)
+            root_span = tracer.start_span("web.request")
+            span = tracer.start_span("web.decoder", child_of=root_span)
 
-        Or if you have a ``Context`` object::
+        Spans from ``start_span`` are not activated by default::
 
-            context = tracer.get_call_context()
-            span = tracer.start_span('web.worker', child_of=context)
+            with tracer.start_span("parent") as parent:
+                assert tracer.current_span() is None
+                with tracer.start_span("child", child_of=parent):
+                    assert tracer.current_span() is None
+
+            new_parent = tracer.start_span("new_parent", activate=True)
+            assert tracer.current_span() is new_parent
+
+        Note: be sure to finish all spans to avoid memory leaks and incorrect
+        parenting of spans.
         """
-        new_ctx = self._check_new_process()
+        if self._new_process:
+            self._new_process = False
 
+            # The spans remaining in the context can not and will not be
+            # finished in this new process. So to avoid memory leaks the
+            # strong span reference (which will never be finished) is replaced
+            # with a context representing the span.
+            if isinstance(child_of, Span):
+                new_ctx = Context(
+                    sampling_priority=child_of.context.sampling_priority,
+                    span_id=child_of.span_id,
+                    trace_id=child_of.trace_id,
+                )
+
+                # If the child_of span was active then activate the new context
+                # containing it so that the strong span referenced is removed
+                # from the execution.
+                if self.context_provider.active() is child_of:
+                    self.context_provider.activate(new_ctx)
+                child_of = new_ctx
+
+        parent = None  # type: Optional[Span]
         if child_of is not None:
             if isinstance(child_of, Context):
-                context = new_ctx or child_of
-                parent = child_of._get_current_span()
+                context = child_of
             else:
                 context = child_of.context
                 parent = child_of
         else:
             context = Context()
-            parent = None
 
         if parent:
-            trace_id = parent.trace_id
-            parent_span_id = parent.span_id
+            trace_id = parent.trace_id  # type: Optional[int]
+            parent_id = parent.span_id  # type: Optional[int]
         else:
             trace_id = context.trace_id
-            parent_span_id = context.span_id
+            parent_id = context.span_id
 
         # The following precedence is used for a new span's service:
         # 1. Explicitly provided service name
@@ -443,12 +506,12 @@ class Tracer(object):
             span = Span(
                 self,
                 name,
+                context=context,
                 trace_id=trace_id,
-                parent_id=parent_span_id,
+                parent_id=parent_id,
                 service=mapped_service,
                 resource=resource,
                 span_type=span_type,
-                _check_pid=False,
                 on_finish=[self._on_span_finish],
             )
 
@@ -456,20 +519,22 @@ class Tracer(object):
             if parent:
                 span.sampled = parent.sampled
                 span._parent = parent
+                span._local_root = parent._local_root
 
+            if span._local_root is None:
+                span._local_root = span
         else:
             # this is the root span of a new trace
             span = Span(
                 self,
                 name,
+                context=context,
                 service=mapped_service,
                 resource=resource,
                 span_type=span_type,
-                _check_pid=False,
                 on_finish=[self._on_span_finish],
             )
-            span.metrics[system.PID] = self._pid or getpid()
-            span.meta["runtime-id"] = get_runtime_id()
+            span._local_root = span
             if config.report_hostname:
                 span.meta[HOSTNAME_KEY] = hostname.get_hostname()
             span.sampled = self.sampler.sample(span)
@@ -499,6 +564,10 @@ class Tracer(object):
                 # We must always mark the span as sampled so it is forwarded to the agent
                 span.sampled = True
 
+        if not span._parent:
+            span.meta["runtime-id"] = get_runtime_id()
+            span.metrics[system.PID] = self._pid
+
         # Apply default global tags.
         if self.tags:
             span.set_tags(self.tags)
@@ -518,8 +587,8 @@ class Tracer(object):
             ):
                 span._set_str_tag(VERSION_KEY, config.version)
 
-        # add it to the current context
-        context._add_span(span)
+        if activate:
+            self.context_provider.activate(span)
 
         # update set of services handled by tracer
         if service and service not in self._services and self._is_span_internal(span):
@@ -531,19 +600,31 @@ class Tracer(object):
         self._hooks.emit(self.__class__.start_span, span)
         return span
 
-    def _on_span_finish(self, span):
-        if not self.enabled:
-            return  # nothing to do
+    start_span = _start_span
 
+    def _on_span_finish(self, span):
+        # type: (Span) -> None
         if self.log.isEnabledFor(logging.DEBUG):
             self.log.debug("finishing span %s (enabled:%s)", span.pprint(), self.enabled)
 
+        active = self.current_span()
+        # Debug check: if the finishing span has a parent and its parent
+        # is not the next active span then this is an error in synchronous tracing.
+        if span._parent is not None and active is not span._parent:
+            self.log.debug(
+                "span %r closing after its parent %r, this is an error when not using async", span, span._parent
+            )
+
+        if not self.enabled:
+            return  # nothing to do
+
         for p in self._span_processors:
             p.on_span_finish(span)
 
     def _initialize_span_processors(self):
         # type: () -> None
         trace_processors = []  # type: List[TraceProcessor]
+        trace_processors += [TraceTagsProcessor()]
         trace_processors += [TraceSamplingProcessor()]
         trace_processors += self._filters
 
@@ -556,42 +637,6 @@ class Tracer(object):
             ),
         ]  # type: List[SpanProcessor]
 
-    def _check_new_process(self):
-        """Checks if the tracer is in a new process (was forked) and performs
-        the necessary updates if it is a new process
-        """
-        pid = getpid()
-        if self._pid == pid:
-            return
-
-        self._pid = pid
-
-        # We have to reseed the RNG or we will get collisions between the processes as
-        # they will share the seed and generate the same random numbers.
-        _rand.seed()
-
-        ctx = self.get_call_context()
-        # The spans remaining in the context can not and will not be finished
-        # in this new process. So we need to copy out the trace metadata needed
-        # to continue the trace.
-        # Also, note that because we're in a forked process, the lock that the
-        # context has might be permanently locked so we can't use ctx.clone().
-        new_ctx = Context(
-            sampling_priority=ctx._sampling_priority,
-            span_id=ctx._parent_span_id,
-            trace_id=ctx._parent_trace_id,
-        )
-        self.context_provider.activate(new_ctx)
-
-        # Assume that the services of the child are not necessarily a subset of those
-        # of the parent.
-        self._services = set()
-
-        # Re-create the background writer thread
-        self.writer = self.writer.recreate()
-        self._initialize_span_processors()
-        return new_ctx
-
     def _log_compat(self, level, msg):
         """Logs a message for the given level.
 
@@ -609,12 +654,9 @@ class Tracer(object):
         else:
             self.log.log(level, msg)
 
-    def trace(self, name, service=None, resource=None, span_type=None):
+    def _trace(self, name, service=None, resource=None, span_type=None):
         # type: (str, Optional[str], Optional[str], Optional[str]) -> Span
-        """
-        Return a span that will trace an operation called `name`. The context that created
-        the span as well as the span parenting, are automatically handled by the tracing
-        function.
+        """Activate and return a new span that inherits from the current active span.
 
         :param str name: the name of the operation being traced
         :param str service: the name of the service being traced. If not set,
@@ -622,49 +664,57 @@ class Tracer(object):
         :param str resource: an optional name of the resource being tracked.
         :param str span_type: an optional operation type.
 
-        You must call `finish` on all spans, either directly or with a context
-        manager::
+        The returned span *must* be ``finish``'d or it will remain in memory
+        indefinitely::
 
-            >>> span = tracer.trace('web.request')
+            >>> span = tracer.trace("web.request")
                 try:
                     # do something
                 finally:
                     span.finish()
 
-            >>> with tracer.trace('web.request') as span:
+            >>> with tracer.trace("web.request") as span:
                     # do something
 
-        Trace will store the current active span and subsequent child traces will
-        become its children::
+        Example of the automatic parenting::
 
-            parent = tracer.trace('parent')     # has no parent span
-            child  = tracer.trace('child')      # is a child of a parent
+            parent = tracer.trace("parent")     # has no parent span
+            assert tracer.current_span() is parent
+
+            child  = tracer.trace("child")
+            assert child.parent_id == parent.span_id
+            assert tracer.current_span() is child
             child.finish()
+
+            # parent is now the active span again
+            assert tracer.current_span() is parent
             parent.finish()
 
-            parent2 = tracer.trace('parent2')   # has no parent span
+            assert tracer.current_span() is None
+
+            parent2 = tracer.trace("parent2")
+            assert parent2.parent_id is None
             parent2.finish()
         """
-
-        # retrieve the Context using the context provider and create
-        # a new Span that could be a root or a nested span
-        context = self.get_call_context()
         return self.start_span(
             name,
-            child_of=context,
+            child_of=self.context_provider.active(),
             service=service,
             resource=resource,
             span_type=span_type,
+            activate=True,
         )
 
+    trace = _trace
+
     def current_root_span(self):
         # type: () -> Optional[Span]
-        """Returns the root span of the current context.
+        """Returns the root span of the current execution.
 
         This is useful for attaching information related to the trace as a
         whole without needing to add to child spans.
 
-        Usage is simple, for example::
+        For example::
 
             # get the root span
             root_span = tracer.current_root_span()
@@ -672,21 +722,21 @@ class Tracer(object):
             if root_span:
                 root_span.set_tag('host', '127.0.0.1')
         """
-        ctx = self.get_call_context()
-        if ctx:
-            return ctx._get_current_root_span()
-        return None
+        span = self.current_span()
+        if span is None:
+            return None
+        return span._local_root
 
     def current_span(self):
         # type: () -> Optional[Span]
+        """Return the active span in the current execution context.
+
+        Note that there may be an active span represented by a context object
+        (like from a distributed trace) which will not be returned by this
+        method.
         """
-        Return the active span for the current call context or ``None``
-        if no spans are available.
-        """
-        ctx = self.get_call_context()
-        if ctx:
-            return ctx._get_current_span()
-        return None
+        active = self.context_provider.active()
+        return active if isinstance(active, Span) else None
 
     def write(self, spans):
         # type: (Optional[List[Span]]) -> None
@@ -720,7 +770,7 @@ class Tracer(object):
         resource=None,  # type: Optional[str]
         span_type=None,  # type: Optional[str]
     ):
-        # type: (...) -> Callable[[Callable[..., Any]], Callable[..., Any]]
+        # type: (...) -> Callable[[AnyCallable], AnyCallable]
         """
         A decorator used to trace an entire function. If the traced function
         is a coroutine, it traces the coroutine execution when is awaited.
@@ -764,6 +814,7 @@ class Tracer(object):
         """
 
         def wrap_decorator(f):
+            # type: (AnyCallable) -> AnyCallable
             # FIXME[matt] include the class name for methods.
             span_name = name if name else "%s.%s" % (f.__module__, f.__name__)
 
@@ -818,18 +869,67 @@ class Tracer(object):
         """
         self.tags.update(tags)
 
+    def _restore_from_shutdown(self):
+        with self._shutdown_lock:
+            if self.start_span is self._start_span:
+                # Already restored
+                return
+
+            atexit.register(self._atexit)
+            forksafe.register(self._child_after_fork)
+
+            self.start_span = self._start_span
+            self.trace = self._trace
+
+            debtcollector.deprecate(
+                "Tracing with a tracer that has been shut down is being deprecated. "
+                "A new tracer should be created for generating new traces",
+                version="1.0.0",
+            )
+
+    def _shutdown_start_span(
+        self,
+        name,  # type: str
+        child_of=None,  # type: Optional[Union[Span, Context]]
+        service=None,  # type: Optional[str]
+        resource=None,  # type: Optional[str]
+        span_type=None,  # type: Optional[str]
+        activate=False,  # type: bool
+    ):
+        # type: (...) -> Span
+        self._restore_from_shutdown()
+
+        return self.start_span(name, child_of, service, resource, span_type, activate)
+
+    def _shutdown_trace(self, name, service=None, resource=None, span_type=None):
+        # type: (str, Optional[str], Optional[str], Optional[str]) -> Span
+        self._restore_from_shutdown()
+
+        return self.trace(name, service, resource, span_type)
+
     def shutdown(self, timeout=None):
         # type: (Optional[float]) -> None
         """Shutdown the tracer.
 
-        This will stop the background writer/worker and flush any finished traces in the buffer.
+        This will stop the background writer/worker and flush any finished traces in the buffer. The tracer cannot be
+        used for tracing after this method has been called. A new tracer instance is required to continue tracing.
 
         :param timeout: How long in seconds to wait for the background worker to flush traces
             before exiting or :obj:`None` to block until flushing has successfully completed (default: :obj:`None`)
         :type timeout: :obj:`int` | :obj:`float` | :obj:`None`
         """
-        self.writer.stop(timeout=timeout)
-        atexit.unregister(self._atexit)
+        try:
+            self.writer.stop(timeout=timeout)
+        except service.ServiceStatusError:
+            # It's possible the writer never got started in the first place :(
+            pass
+
+        with self._shutdown_lock:
+            atexit.unregister(self._atexit)
+            forksafe.unregister(self._child_after_fork)
+
+            self.start_span = self._shutdown_start_span  # type: ignore[assignment]
+            self.trace = self._shutdown_trace  # type: ignore[assignment]
 
     @staticmethod
     def _use_log_writer():
